/*
 * Hormann, Leydold, Derflinger
 * Automatic Nonuniform Random Variate Generation
 * Springer, Heidelberg, New York, 2003
 * http://statistik.wu-wien.ac.at/arvag
 * 
 * Code for Algorithm 10.1 (Indexed-Search-Tail), p. 217
 *
 * this file: algo03_02.c
 *
 * For a demo how to use this algorithm see the file "demo_algo03_02.c".
 *
 */

#include <math.h>
#include <stdlib.h>
#include <stdio.h>
#include "uniform.h"

  static int *gt=NULL,Ch;
  static double *pc=NULL;


int setup03_02(double (*pmf)(int), int L, int C)
/*
 * Algorithm 3.2 (Indexed-Search), p. 46  
 * generates discrete variates
 *
 * setup ... 1 = new setup; 0 = sample; -1 = free tables
 * pmf... function pointer to probability (mass) function for k >= 0
 * L ... non zero probabilities assumed for (0, 1, 2, ..., L-1)
 * C ... length of guide table
 *
 */
{ double psum=0.;
  int i,j;

      if(pc!=NULL){
	free(pc);
	free(gt);
      }
      pc = malloc(L*sizeof(double));
      gt = malloc(C*sizeof(int));
      Ch = C;
/*1*/ for( i=0; i<L; i++){
        psum+=(*pmf)(i);
        pc[i]=psum;
      }
/*2*/ gt[0]=0;  
      i=0;
/*3*/ for(j=1; j<C; j++){
/*4*/   while(j/(double)C > pc[i]) 
/*5*/     i = i+1;
/*6*/   gt[j] = i;
      }
      return 1;
}

int generator03_02()
/* is called after the setup to generate random variates */
{  double u;
   int x;
  
      if(pc==NULL){
	printf("algo03_02: sampling invoked without setup!! run setup first\n");
	return -9999;
      }
/*7*/ u = uniform();
/*8*/ x = gt[(int)(u*Ch)];
/*9*/ while(u > pc[x]) 
/*10*/  x++;
/*10*/return x;
}

int free03_02()
/* to free the allocated memory*/
{
    free(pc);
    pc = NULL;
    free(gt);
    gt = NULL;
    return 1;
}



