/*
 * Hormann, Leydold, Derflinger
 * Automatic Nonuniform Random Variate Generation
 * Springer, Heidelberg, New York, 2003
 * http://statistik.wu-wien.ac.at/arvag/monograph
 * 
 * Code for Algorithm 9.3 (Hazard Rate Thinning), p. 196 
 *
 * this file: algo09_03.c
 *
 * For a demo how to use this algorithm see the file "demo_algo09_03.c".
 *
 */

#include <math.h>
#include <stdlib.h>
#include <stdio.h>
#include "uniform.h"

double generator09_03(double (*h)(double),double (*g)(double),
                      double (*icg)(double))
/*
 * Algorithm 9.3 (Hazard Rate Thinning), p. 196 
 *
 * h   ... function pointer to the hazard rate h(x)
 * g   ... function pointer to the dominatig hazard rate g(x)
 * icg ... function pointer to the inverse cumulative hazard rate of g(x)
 */
{ 
      double x, e,v;
      int i;  

/*1*/ x = 0.;
      e = 0.;
      i = 1;
/*2*/ while(1){
/*3*/	e = e-log(1.-uniform());
	x = (*icg)(e);
/*4*/	v = (*g)(x)*uniform();
/*5*/	if(v <= (*h)(x)) 
/*6*/     return x;
/*7*/   else
/*8*/	  i++;
      }
}



