/*
 * Hormann, Leydold, Derflinger
 * Automatic Nonuniform Random Variate Generation
 * Springer, Heidelberg, New York, 2003
 * http://statistik.wu-wien.ac.at/arvag/monograph
 * 
 * Demo how to use Algorithm 4.3 (TDR immediate acceptance), p. 92  
 *
 * this file: demo_algo04_03.c
 *
 * to compile this file you can use the command:
 *
 * cc -o algo04_03 demo_algo04_03.c algo04_03.c uniform.c statutil.c -lm
 *
 */

#include <math.h>
#include <stdlib.h>
#include <stdio.h>
#include "algo.h"
#include "statutil.h"
#include "uniform.h"


/* PDF of the distribution and its derivative (dpdf)
   you can insert your own distribution here
*/

double my_pdf(double x)
{return 2.*x;}
/* triangular on (0,1) */

double my_dpdf(double x)
/* derivative of the triangular pdf on (0,1) */
{ 
  if( x >= 0. && x <= 1.)
    return 2.;
  else return 0.;
}




#define CLAS 500 /*sets number of classes for the chi-square test*/

static long int obs[CLAS];

int main()
/* demonstrates the use of the algorithm
   and conducts a chi-square test */
{ int i;
 double x,cp[5];

 setseed(13324821); /*set seed for uniform generator*/

 printf("Demo for Algorithm 4.3\nfirst we display ten numbers generated from my_pdf()\n");
 cp[0] = 0.1;
 cp[1] = 0.3;
 cp[2] = 0.5;
 cp[3] = 0.7;
 cp[4] = 0.9;
 setup04_03(my_pdf,my_dpdf,0.,1.,5,cp);/* calls set-up*/
 for(i=0;i<10;i++){
   x = generator04_03();
      /* it is not necessary to specify the inputs again */
   printf("x %f\n",x);
 }
 free04_03();/* to clear the memory */

 printf("\nNow we make a chi-square test with sample size %d for  the normal distribution\n\n", CLAS*CLAS);
 for(i=0;i<CLAS;i++) obs[i]=0;
 cp[0] = -3.;
 cp[1] = -1.;
 cp[2] = 0.01;
/* note: the mode (0) is not allowed for this simple 
   implementation as this would lead to divison by zero */
 cp[3] = 1.;
 cp[4] = 3.;
 setup04_03(pdf_normal,dpdf_normal,-10.,10.,5,cp);/* calls set-up*/
 for(i=0;i<CLAS*CLAS;i++){
   x = generator04_03();
   obs[(int)(cdf_normal(x)*CLAS)]++;
 }
 free04_03();/* to clear the memory */

 chi2test(obs,CLAS,CLAS*CLAS,1); 
 printf("\n");
 return 0;
}

