/*
 * Hormann, Leydold, Derflinger
 * Automatic Nonuniform Random Variate Generation
 * Springer, Heidelberg, New York, 2003
 * http://statistik.wu-wien.ac.at/arvag/monograph
 * 
 * Demo how to use Algorithm 9.6 (Increasing Hazard Rate dynamic composition) p.200 
 *
 * this file: demo_algo09_06.c
 *
 * to compile this file you can use the command:
 *
 * cc -o algo09_06 demo_algo09_06.c algo09_06.c uniform.c statutil.c -lm
 *
 */

#include <math.h>
#include <stdlib.h>
#include <stdio.h>
#include "algo.h"
#include "statutil.h"
#include "uniform.h"


/* Inverse cumulative hazard rate of the distribution
   you can insert your own inverse cumulative hazard rate here
*/
double my_hazard(double x)
/* hazard rate of the Weibull distribution with alpha = 2. beta = 2.5 */
{ return 2.*2.5*exp(log(x)*(2.5-1.));
}




#define CLAS 500 /*sets number of classes for the chi-square test*/

static long int obs[CLAS];

int main()
/* demonstrates the use of the algorithm
   and conducts a chi-square test */
{ int i;
 double x;

 setseed(1766345); /*set seed for uniform generator*/

 printf("Demo for Algorithm 9.6\nfirst we display ten numbers generated from my_hazard()\n\n");
 for(i=0;i<10;i++){
   x = generator09_06(my_hazard,1.);
   printf("x %f\n",x);
 }
 
 printf("\nNow we make a chi-square test with sample size %d \n\n",CLAS*CLAS);
 for(i=0;i<CLAS;i++) obs[i]=0;

 for(i=0;i<CLAS*CLAS;i++){
   x = generator09_06(my_hazard,1.);
   obs[(int)(cdf_weibull(x,2.,2.5)*CLAS)]++;
 }

 chi2test(obs,CLAS,CLAS*CLAS,1); 
 printf("\n");
 return 0;
}


