/*
 * Hormann, Leydold, Derflinger
 * Automatic Nonuniform Random Variate Generation
 * Springer, Heidelberg, New York, 2003
 * http://statistik.wu-wien.ac.at/arvag/monograph
 * 
 * Demo how to use Algorithm 10.2 (Indexed-Search-2-Tails), p. 220
 *
 * this file: demo_algo10_02.c
 *
 * to compile this file you can use the command:
 *
 * cc -o algo10_02 demo_algo10_02.c algo10_02.c uniform.c -lm
 *
 */

#include <math.h>
#include <stdlib.h>
#include <stdio.h>
#include "algo.h"
#include "uniform.h"


/* PMF of the distribution
   you can insert your own distribution here
*/
#define PGEOM 0.7
#define MODE 100 
/* MODE must be chosen big, that the probability for negative values is close to 0*/

double my_pmf(int i)
     /* "double geometric" with PGEOM and MODE*/
{ int mode=MODE;
 if(i>mode) i= i-mode;
 else i = mode - i +1;
return 0.5*(i>0 ? PGEOM*exp((i-1.)*log(1.-PGEOM)) : 0.);
}




#define CLAS 150 /*set numberq of integers for which frequency is counted*/
#define N 100000 /* sample size */

static long int obs[CLAS];

int main()
/* demonstrates the use of the algorithm
   and compares probabilities to observed relative frequencies */
{ int i,k;
  double p;

 setseed(133248); /*set seed for uniform generator*/


 printf("Demo for Algorithm 10.1\nfirst we display ten numbers generated from my_pmf()\n");
 setup10_02(my_pmf, MODE-1,MODE+1,100);/* calls the setup*/
 for(i=0;i<10;i++){
   k =  generator10_02();/* calls the sampling*/ 
   /* it is not necessary to specify the inputs again */
   printf("k %d\n",k);
 }
 
 printf("\nNow we count the frequencies with sample size %d\n\n", N);
 for(i=0;i<CLAS;i++) obs[i]=0;

 for(i=0;i<N;i++){
   k =  generator10_02();/* calls the sampling*/ 
   if(k<CLAS) obs[k]++;
 }


 printf("  k    Probability(k)   Observed relative frequency    ratio\n");
 for( i=0; i<CLAS; i++){
   p = my_pmf(i);
   if(obs[i]>0)
     printf(" %2d      %f              %f              %f\n",
	     i,       p, obs[i]/(double)N, p/(obs[i]/(double)N));
}

 printf("\n");

 free10_02();/* to free the allocated memory*/ 

 return 0;
}
