/*
 * Hormann, Leydold, Derflinger
 * Automatic Nonuniform Random Variate Generation
 * Springer, Heidelberg, New York, 2003
 * http://statistik.wu-wien.ac.at/arvag/monograph
 * 
 * Demo how to use Algorithm 10.6 (Discrete-Simple-RoU), p. 231
 *
 * this file: demo_algo10_06.c
 *
 * to compile this file you can use the command:
 *
 * cc -o algo10_06 demo_algo10_06.c algo10_06.c uniform.c statutil.c -lm
 *
 */

#include <math.h>
#include <stdlib.h>
#include <stdio.h>
#include "algo.h"
#include "uniform.h"
#include "statutil.h"


/* PMF of the distribution
   you can insert your own distribution here
*/
#define MU 10.

double my_pmf(int k)
{
 if(k<0) return 0.;  
 return exp(-MU+k*log(MU)-lnkfakt(k));
}
/*************************************************************************/




#define CLAS 150 /*set numberq of integers for which frequency is counted*/
#define N 100000 /* sample size */

static long int obs[CLAS];

int main()
/* demonstrates the use of the algorithm
   and compares probabilities to observed relative frequencies */
{ int i,k,m;
  double p;

 setseed(133248); /*set seed for uniform generator*/


 printf("Demo for Algorithm 10.6\nfirst we display ten numbers generated from my_pmf()\n");
 m = (int)MU;
 setup10_06(my_pmf, m, 1., -1.);/* calls the setup*/
 /* setup10_06(my_pmf, m, 1., 0.45793);only correct for MU = 10. */
 for(i=0;i<10;i++){
   k =  generator10_06();/* calls the sampling*/ 
   /* it is not necessary to specify the inputs again */
   printf("k %d\n",k);
 }
 
 printf("\nNow we count the frequencies with sample size %d\n\n", N);
 for(i=0;i<CLAS;i++) obs[i]=0;

 for(i=0;i<N;i++){
   k =  generator10_06();/* calls the sampling*/ 
   if(k<CLAS) obs[k]++;
 }


 printf("  k    Probability(k)   Observed relative frequency    ratio\n");
 for( i=0; i<CLAS; i++){
   p = my_pmf(i);
   if(obs[i]>0)
     printf(" %2d      %f              %f              %f\n",
	     i,       p, obs[i]/(double)N, p/(obs[i]/(double)N));
}

 printf("\n");

 return 0;
}
