/*
 * Hormann, Leydold, Derflinger
 * Automatic Nonuniform Random Variate Generation
 * Springer, Heidelberg, New York, 2003
 * http://statistik.wu-wien.ac.at/arvag/monograph
 * 
 * Demo how to use Algorithm 11.6 (Uniform-Sphere, p. 253)
 *
 * this file: demo_algo11_06.c
 *
 * to compile this file you can use the command:
 *
 * cc -o algo11_06 demo_algo11_06.c algo11_06.c algo02_11.c uniform.c statutil.c -lm
 *
 */

#include <math.h>
#include <stdlib.h>
#include <stdio.h>
#include "algo.h"
#include "statutil.h"
#include "uniform.h"

#ifndef M_PI
#define M_PI       3.14159265358979323846264338328      /* pi */
#endif

#define CLAS 500

static long int obs[CLAS];

int main()
{ int i,j,d;
 double x[10];

 setseed(13324821); /*set seed for uniform generator*/

 printf("Demo for Algorithm 11.6\n");
 printf("first we display ten generated vectors for dimension 6\n");
 d=6;
 for(i=0;i<10;i++){
   generator11_06(6,x);
   for(j=1;j<d;j++) printf("%8.3f, ",x[j]);
   printf("%8.3f\n",x[d]);
 }


 printf("Now we test the uniform distribution of the angle for dimension 2\n");

 for(i=0;i<CLAS;i++) obs[i]=0;

 for(i=0;i<CLAS*CLAS;i++){
   generator11_06(2,x);
   obs[(int)((asin(x[1])+M_PI*0.5)/M_PI*CLAS)]++;
 }
 chi2test(obs,CLAS,CLAS*CLAS,1); 
 

 return 0;
}

