/*
 * Hormann, Leydold, Derflinger
 * Automatic Nonuniform Random Variate Generation
 * Springer, Heidelberg, New York, 2003
 * http://statistik.wu-wien.ac.at/arvag
 * 
 * Code for Example 2.7, p. 19, 
 * "half normal distribution by rejection from exponential hat-function 
 *
 * this file: example02_07.c
 *
 * to compile this file you can use the command:
 *
 * cc -o example02_07 example02_07.c uniform.c statutil.c -lm
 *
 */


#include <math.h>
#include <stdlib.h>
#include <stdio.h>
#include "statutil.h"
#include "uniform.h"

/**********************************************************************/

double example02_07(void)
/* 
 * generates random variates from the positive half normal distribution 
 * by rejection from an exponential hat-function 
 */
{ double u, v, x;

/*1*/   while(1){ 
/*2*/     u = uniform();
/*3*/     x = -log(1.-u);
/*4*/     v = uniform();
/*5+6*/ if(log(v)<=x-x*x*0.5-0.5) return x;
 }
}

/*****************************************************************/


#define CLAS 500

static long int obs[CLAS*CLAS];

int main()
{ int i;
 double x;

 setseed(13324885); /*set seed for uniform generator*/

 printf("Example 2.7\nfirst we display ten generated numbers\n");

 for(i=0;i<10;i++){
   x = example02_07();
   printf("x %f\n",x);
 }
 
 printf("\nNow we make a chi-square test with sample size %d\n\n", CLAS*CLAS);
 for(i=0;i<CLAS;i++) obs[i]=0;

 for(i=0;i<CLAS*CLAS;i++){
   x = example02_07();
   obs[(int)(cdf_halfnorm(x)*CLAS)]++;
 }

 chi2test(obs,CLAS,CLAS*CLAS,1); 
 printf("\n");
 return 0;
}






